@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	3 October 1995
- * Last Edited:	10 November 2004
+ * Last Edited:	8 March 2005
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 1988-2004 University of Washington.
+ * Copyright 1988-2005 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -920,6 +920,7 @@ long mbx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   else {
     mm_critical (dstream);	/* go critical */
     fstat (fd,&sbuf);		/* get current file size */
+    errno = 0;
     for (ret = LONGT; ret && message; ) {
       if (!SIZE (message)) {	/* guard against zero-length */
 	mm_log ("Append of zero-length message",ERROR);
@@ -952,8 +953,10 @@ long mbx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 				/* revert file */
       ftruncate (fd,sbuf.st_size);
       close (fd);		/* make sure fclose() doesn't corrupt us */
-      sprintf (tmp,"Message append failed: %s",strerror (errno));
-      mm_log (tmp,ERROR);
+      if (errno) {
+	sprintf (tmp,"Message append failed: %s",strerror (errno));
+	mm_log (tmp,ERROR);
+      }
       ret = NIL;
     }
     if (ret) times.actime = time (0) - 1;
