@@ -1,5 +1,5 @@
 /*
- * $Id: edef.h,v 4.33 2004/03/26 22:34:19 hubert Exp $
+ * $Id: edef.h,v 4.34 2004/10/14 01:27:01 hubert Exp $
  *
  * Program:	Global definitions and initializations
  *
@@ -85,6 +85,7 @@ BUFFER  *blistp;                /* Buffer for C-X C-B           */
 BUFFER  *bfind PROTO((char *, int, int)); /* Lookup a buffer by name */
 LINE    *lalloc PROTO((int));   /* Allocate a line              */
 int	 km_popped;		/* menu popped up               */
+int	 panicking;		/* we are currently panicking	*/
 #if	defined(USE_TERMCAP) || defined(USE_TERMINFO) || defined(VMS)
 KBESC_T *kbesc;			/* keyboard esc sequence trie   */
 #endif /* USE_TERMCAP/USE_TERMINFO/VMS */
@@ -140,6 +141,7 @@ extern  BUFFER  *blistp;                /* Buffer for C-X C-B           */
 extern  BUFFER  *bfind PROTO((char *, int, int)); /* Lookup a buffer by name */
 extern  LINE    *lalloc PROTO((int));   /* Allocate a line              */
 extern  int	 km_popped;		/* menu popped up               */
+extern  int	 panicking;		/* we are currently panicking	*/
 /*
  * This is a weird one. It has to be defined differently for pico and for
  * pine. It seems to need to be defined at startup as opposed to set later.
