@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.84 2004/05/07 21:41:10 jpf Exp $";
+static char rcsid[] = "$Id: file.c,v 4.85 2004/06/11 21:49:40 hubert Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -253,7 +253,7 @@ int f, n;
 
 		    fname[0] = '\0';
 		    if((s = FileBrowse(dir, NLINE, fname, NLINE, 
-				       NULL, FB_READ)) == 1){
+				       NULL, FB_READ, NULL)) == 1){
 			if(gmode&MDSCUR){
 			    emlwrite("Can't insert in restricted mode",
 				     NULL);
@@ -640,7 +640,8 @@ int f, n;
 			    : gethomedir(NULL)));
 
 		strcpy(origshows, shows);
-		if ((s = FileBrowse(shows, NLINE, fname, NFILEN, NULL, FB_SAVE)) == 1) {
+		if ((s = FileBrowse(shows, NLINE, fname, NFILEN, NULL,
+				    FB_SAVE, NULL)) == 1) {
 		  if (strlen(shows)+strlen(S_FILESEP)+strlen(fname) < NLINE){
 		    strcat(shows, S_FILESEP);
 		    strcat(shows, fname);
