@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pilot.c,v 4.22 2004/05/07 21:43:40 jpf Exp $";
+static char rcsid[] = "$Id: pilot.c,v 4.23 2004/06/11 21:49:40 hubert Exp $";
 #endif
 /*
  * Program:	Main stand-alone Pine File Browser routines
@@ -134,7 +134,7 @@ char    *argv[];
       emlwrite("Checking for new mail every %D seconds", (void *) timeo);
 
     set_browser_title(PILOT_VERSION);
-    FileBrowse(filedir, NSTRING, filename, NSTRING, NULL, 0);
+    FileBrowse(filedir, NSTRING, filename, NSTRING, NULL, 0, NULL);
     wquit(1, 0);
 }
 
