@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: search.c,v 4.31 2004/05/07 21:45:04 jpf Exp $";
+static char rcsid[] = "$Id: search.c,v 4.32 2004/07/01 21:33:30 jpf Exp $";
 #endif
 /*
  * Program:	Searching routines
@@ -230,7 +230,7 @@ forwsearch(f, n)
 	    mlerase();
 	    FWS_RETURN(TRUE);
 
-	  case (CTRL|'J'):
+	  case (CTRL|'U'):
 	    fillbuf(0, 1);
 	    mlerase();
 	    FWS_RETURN(TRUE);
@@ -532,9 +532,9 @@ int   repl_mode;
 	    menu_pat[i].key    = (CTRL|'O');
 	    KS_OSDATASET(&menu_pat[i], KS_NONE);
 
-	    menu_pat[++i].name = "^J";
+	    menu_pat[++i].name = "^U";
 	    menu_pat[i].label  = "FullJustify";
-	    menu_pat[i].key    = (CTRL|'J');
+	    menu_pat[i].key    = (CTRL|'U');
 	    KS_OSDATASET(&menu_pat[i], KS_NONE);
 	}
 
@@ -658,9 +658,9 @@ int   text_mode;
 	    menu_pat[i].key    = (CTRL|'O');
 	    KS_OSDATASET(&menu_pat[i], KS_NONE);
 
-	    menu_pat[++i].name = "^J";
+	    menu_pat[++i].name = "^U";
 	    menu_pat[i].label  = "FullJustify";
-	    menu_pat[i].key    = (CTRL|'J');
+	    menu_pat[i].key    = (CTRL|'U');
 	    KS_OSDATASET(&menu_pat[i], KS_NONE);
 	}
 
