@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.269 2004/10/14 01:41:02 hubert Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.270 2005/01/22 01:28:19 jpf Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -1526,6 +1526,7 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
     MAILSTREAM *ourstream = NULL;
 
     if(check_for_move_mbox(mailbox, source, sizeof(source), &target)){
+	DRIVER *d;
 
 	memset(&status, 0, sizeof(status));
 	memset(&cache,  0, sizeof(cache));
@@ -1539,6 +1540,7 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
 
 	stream = sp_stream_get(target, SP_SAME);
 
+	/* should never be news, don't worry about mulnewrsc flag*/
 	if((ret = pine_mail_status_full(stream, target, flags, uidvalidity,
 					uidnext))
 	   && !status.recent){
@@ -1560,6 +1562,11 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
 		    stream = pine_mail_open(NULL, source, openflags, NULL);
 		    ourstream = stream;
 		}
+		else if(F_ON(F_ENABLE_MULNEWSRCS, ps_global)
+			&& d && (!strucmp(d->name, "news")
+				 || !strucmp(d->name, "nntp")))
+		  flags |= SA_MULNEWSRC;
+
 	    }
 
 	    if(mail_status(stream, source, flags)){
@@ -1757,6 +1764,10 @@ pine_mail_status_full(stream, mailbox, flags, uidvalidity, uidnext)
 		    ourstream = stream;
 		}
 	    }
+	    else if(F_ON(F_ENABLE_MULNEWSRCS, ps_global)
+		    && d && (!strucmp(d->name, "news")
+			     || !strucmp(d->name, "nntp")))
+	      flags |= SA_MULNEWSRC;
 	}
 
 	ret = mail_status(stream, mailbox, flags);	/* non #move case */
