@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.138 2005/01/13 22:12:06 hubert Exp $";
+static char rcsid[] = "$Id: status.c,v 4.139 2005/04/26 22:14:46 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -1355,7 +1355,8 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
     for(i = 0; esc_list && esc_list[i].ch != -1 && i < 11; i++){
       if(esc_list[i].ch == -2) /* -2 means to skip this key and leave blank */
 	continue;
-      max_label = max(max_label, strlen(esc_list[i].name));
+      if(esc_list[i].name)
+        max_label = max(max_label, strlen(esc_list[i].name));
     }
 
     maxcol = ps_global->ttyo->screen_cols - max_label - 1;
@@ -1413,7 +1414,8 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 	 * the right.  A key is invisible if it's label is "".
 	 */
 	if(i >= 12){
-	    if(esc_list[i-start].label[0] != '\0'){  /* visible */
+	    if(esc_list[i-start].label
+	       && esc_list[i-start].label[0] != '\0'){  /* visible */
 		if(i == 12){  /* special case where we put it in help slot */
 		    if(help_text != NO_HELP)
 		  panic("Programming botch in radio_buttons(): too many keys");
@@ -1423,7 +1425,9 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 
 		    fkey_table[0] = esc_list[i-start].ch;
 		    rb_keymenu.keys[0].name  = esc_list[i-start].name;
-		    if(esc_list[i-start].rval == dflt){
+		    if(esc_list[i-start].ch != -2
+		       && esc_list[i-start].rval == dflt
+		       && esc_list[i-start].label){
 			ds = (char *)fs_get((strlen(esc_list[i-start].label)+3)
 					    * sizeof(char));
 			sprintf(ds, "[%s]", esc_list[i-start].label);
@@ -1442,7 +1446,9 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 
 	    fkey_table[i] = esc_list[i-start].ch;
 	    rb_keymenu.keys[i].name  = esc_list[i-start].name;
-	    if(esc_list[i-start].rval == dflt){
+	    if(esc_list[i-start].ch != -2
+	       && esc_list[i-start].rval == dflt
+	       && esc_list[i-start].label){
 		ds = (char *)fs_get((strlen(esc_list[i-start].label) + 3)
 				    * sizeof(char));
 		sprintf(ds, "[%s]", esc_list[i-start].label);
@@ -1535,7 +1541,7 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 
 		  MoveCursor(real_line,len=min(RAD_BUT_COL+strlen(q),maxcol+1));
 		  for(n = 0, len = ps_global->ttyo->screen_cols - len;
-		      esc_list[i].label[n] && len > 0;
+		      esc_list[i].label && esc_list[i].label[n] && len > 0;
 		      n++, len--)
 		    Writechar(esc_list[i].label[n], 0);
 
@@ -1557,9 +1563,10 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 	      if(ch == esc_list[i].rval){
 		  int len, n;
 
-		  MoveCursor(real_line, len=min(RAD_BUT_COL+strlen(q),maxcol+1));
+		  MoveCursor(real_line,len=min(RAD_BUT_COL+strlen(q),maxcol+1));
 		  for(n = 0, len = ps_global->ttyo->screen_cols - len;
-		      esc_list[i].label[n] && len > 0; n++, len--)
+		      esc_list[i].label && esc_list[i].label[n] && len > 0;
+		      n++, len--)
 		    Writechar(esc_list[i].label[n], 0);
 		  break;
 	      }
@@ -1709,12 +1716,18 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 
 
 #define OTHER_RETURN_VAL 1300
+#define KEYS_PER_LIST 8
 
 /*
  * This should really be part of radio_buttons itself, I suppose. It was
  * easier to do it this way. This is for when there are more than 12
  * possible commands. We could have all the radio_buttons calls call this
  * instead of radio_buttons, or rename this to radio_buttons.
+ *
+ * Radio_buttons is limited to 10 visible commands unless there is no Help,
+ * in which case it is 11 visible commands.
+ * Double_radio_buttons is limited to 16 visible commands because it uses
+ * slots 3 and 4 for readability and the OTHER CMD.
  */
 int
 double_radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text,
@@ -1752,21 +1765,21 @@ double_radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text,
     /*
      * Won't fit, split it into two lists.
      *
-     * We can fit at most 9 items in the visible list. The rest of
+     * We can fit at most 8 items in the visible list. The rest of
      * the commands have to be invisible. Each of list1 and list2 should
-     * have no more than 9 visible (name != "" || label != "") items.
+     * have no more than 8 visible (name != "" || label != "") items.
      */
-    list1 = (ESCKEY_S *)fs_get(10 * sizeof(*list1));
-    memset(list1, 0, 10 * sizeof(*list1));
-    list2 = (ESCKEY_S *)fs_get(10 * sizeof(*list2));
-    memset(list2, 0, 10 * sizeof(*list2));
+    list1 = (ESCKEY_S *)fs_get((KEYS_PER_LIST+1) * sizeof(*list1));
+    memset(list1, 0, (KEYS_PER_LIST+1) * sizeof(*list1));
+    list2 = (ESCKEY_S *)fs_get((KEYS_PER_LIST+1) * sizeof(*list2));
+    memset(list2, 0, (KEYS_PER_LIST+1) * sizeof(*list2));
 
-    for(j=0,i=0; esc_list[i].ch != -1 && j < 9; j++,i++)
+    for(j=0,i=0; esc_list[i].ch != -1 && j < KEYS_PER_LIST; j++,i++)
       list1[j] = esc_list[i];
     
     list1[j].ch = -1;
 
-    for(j=0; esc_list[i].ch != -1 && j < 9; j++,i++)
+    for(j=0; esc_list[i].ch != -1 && j < KEYS_PER_LIST; j++,i++)
       list2[j] = esc_list[i];
 
     list2[j].ch = -1;
@@ -1801,32 +1814,40 @@ construct_combined_esclist(list1, list2)
     ESCKEY_S *list;
     int       i, j=0, count;
     
-    count = 1;	/* for OTHER key */
+    count = 2;	/* for blank key and for OTHER key */
     for(i=0; list1 && list1[i].ch != -1; i++)
       count++;
     for(i=0; list2 && list2[i].ch != -1; i++)
       count++;
     
-    list = (ESCKEY_S *)fs_get((count + 1) * sizeof(*list));
+    list = (ESCKEY_S *) fs_get((count + 1) * sizeof(*list));
     memset(list, 0, (count + 1) * sizeof(*list));
 
+    list[j].ch    = -2;			/* leave blank */
+    list[j].rval  = 0;
+    list[j].name  = NULL;
+    list[j++].label = NULL;
+
     list[j].ch    = 'o';
     list[j].rval  = OTHER_RETURN_VAL;
     list[j].name  = "O";
     list[j].label = "OTHER CMDS";
 
-    /* just checking */
+    /*
+     * Make sure that O for OTHER CMD or the return val for OTHER CMD
+     * isn't used for something else.
+     */
     for(i=0; list1 && list1[i].ch != -1; i++){
 	if(list1[i].rval == list[j].rval)
 	  panic("1bad rval in d_r");
-	if(list1[i].ch == list[j].ch)
+	if(F_OFF(F_USE_FK,ps_global) && list1[i].ch == list[j].ch)
 	  panic("1bad ch in ccl");
     }
 
     for(i=0; list2 && list2[i].ch != -1; i++){
 	if(list2[i].rval == list[j].rval)
 	  panic("2bad rval in d_r");
-	if(list2[i].ch == list[j].ch)
+	if(F_OFF(F_USE_FK,ps_global) && list2[i].ch == list[j].ch)
 	  panic("2bad ch in ccl");
     }
 
@@ -1834,9 +1855,10 @@ construct_combined_esclist(list1, list2)
 
     /* the visible set */
     for(i=0; list1 && list1[i].ch != -1; i++){
-	list[j++] = list1[i];
-	if(i > 9 && list1[i].label[0] != '\0')
+	if(i >= KEYS_PER_LIST && list1[i].label[0] != '\0')
 	  panic("too many visible keys in ccl");
+
+	list[j++] = list1[i];
     }
 
     /* the rest are invisible */
